import 'dart:async';
import 'dart:typed_data';
import 'dart:ui' as ui;
import 'dart:io';

// import 'package:fitton/screens/skinColourDetection/imagepicker.dart';
import 'package:fitton/component/clipper.dart';
import 'package:fitton/component/default_button.dart';
import 'package:fitton/screens/skinColourDetection/imagepicker.dart';
import 'package:fitton/screens/suggestColor/suggestColor.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:image/image.dart' as img;

import 'package:flutter/services.dart' show rootBundle;
import 'package:image_picker/image_picker.dart';

class ColorPickerWidget extends StatefulWidget {
  final File iPath;
  ColorPickerWidget(this.iPath);
  @override
  _ColorPickerWidgetState createState() => _ColorPickerWidgetState( );
}

class _ColorPickerWidgetState extends State<ColorPickerWidget> {

  GlobalKey imageKey = GlobalKey();
  GlobalKey paintKey = GlobalKey();

  // CHANGE THIS FLAG TO TEST BASIC IMAGE, AND SNAPSHOT.
  bool useSnapshot = true;

  // based on useSnapshot=true ? paintKey : imageKey ;
  GlobalKey currentKey;

  final StreamController<Color> _stateController = StreamController<Color>();
  img.Image photo;

  @override
  void initState() {
    currentKey = useSnapshot ? paintKey : imageKey;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // final String title = useSnapshot ? "snapshot" : "basic";
    return Scaffold(
      body: Stack(
        children:[
          ClipPath(
          clipper: MyClipper(),
          child: Container(
            color: Colors.deepPurpleAccent,
          )
        ),
        StreamBuilder(
            initialData: Colors.white,
            stream: _stateController.stream,
            builder: (buildContext, snapshot) {
              Color selectedColor = snapshot.data ?? Colors.white;
              return SingleChildScrollView(
                child: Column(
                  children: <Widget>[
                    SizedBox(
                      height: 90.0,
                    ),
                    Center(
                      child: Text("Let's Find Your Skin Tone", style: TextStyle(color: Colors.white, fontSize: 30, fontWeight: FontWeight.bold),),
                    ),
                    SizedBox(
                      height: 60.0,
                    ),
                    Padding(
                        padding: EdgeInsets.only(left:30.0, right: 30.0 ),
                        child:RepaintBoundary(
                        key: paintKey,
                        child: GestureDetector(
                          onPanDown: (details) {
                            searchPixel(details.globalPosition);
                          },
                          onPanUpdate: (details) {
                            searchPixel(details.globalPosition);
                          },
                          child: Center(
                            child: Image.file(
                              widget.iPath,
                              key: imageKey,
                              fit: BoxFit.fill,
                            ),
                          ),
                        ),
                      ),
                    ),
                    SizedBox(
                      height: 50.0,
                    ),
                    Padding(
                      padding: EdgeInsets.only(left: 30.0, right: 30.0),
                      child:
                      Container(
                          child: Row(
                            children: [
                              Column(
                                children: <Widget>[
                                  Container(
                                    width: 230,
                                    height: 50,
                                    child: Center(
                                      child: Text(selectedColor == Colors.white ? "Wait, Data is loading" : "Your Skin Tone" ,style: TextStyle(color: Colors.deepPurpleAccent, fontSize: 22,),),
                                    ),
                                  )
                                ],
                              ),
                              SizedBox(
                                width: 30.0,
                              ),
                              Column(
                                children: <Widget>[
                                  Container(
                                    width: 100,
                                    height: 50,
                                    decoration: BoxDecoration(
                                        shape: BoxShape.rectangle,
                                        borderRadius: BorderRadius.circular(20.0),
                                        color: selectedColor,
                                        border: Border.all(width: 1.0, color: Colors.deepPurpleAccent,),
                                        boxShadow: [
                                          BoxShadow(
                                              color: Colors.black12,
                                              blurRadius: 4,
                                              offset: Offset(0, 2))
                                        ]),
                                  ),
                                ],
                              ),
                            ],
                          )
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.only(left: 30.0, right: 30.0),
                      child: Column(
                          children: <Widget>[
                            SizedBox(
                              height: 20.0,
                            ),
                            DefaultButton(
                              text: "Go Ahead ✨",
                              bg: false,
                              press: selectedColor == Colors.white ? null : () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(builder: (context) => (SuggestColor(selectedColor))),
                                );
                              },
                            ),
                            SizedBox(
                              height: 20.0,
                            ),
                            DefaultButton(
                              text: "Back",
                              bg: true,
                              press: () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(builder: (context) => (ImgPicker())),
                                );
                              },
                            ),
                            SizedBox(
                              height: 20.0,
                            ),
                          ]
                      ),
                    ),
                  ],
                ),
              );
            }),
        ])
    );
  }

  void searchPixel(Offset globalPosition) async {
    if (photo == null) {
      await (useSnapshot ? loadSnapshotBytes() : loadImageBundleBytes());
    }
    _calculatePixel(globalPosition);
  }

  void _calculatePixel(Offset globalPosition) {
    RenderBox box = currentKey.currentContext.findRenderObject();
    Offset localPosition = box.globalToLocal(globalPosition);

    double px = localPosition.dx;
    double py = localPosition.dy;

    if (!useSnapshot) {
      double widgetScale = box.size.width / photo.width;
      print(py);
      px = (px / widgetScale);
      py = (py / widgetScale);
    }

    int pixel32 = photo.getPixelSafe(px.toInt(), py.toInt());
    int hex = abgrToArgb(pixel32);

    _stateController.add(Color(hex));
  }

  Future<void> loadImageBundleBytes() async {
    ByteData imageBytes = await rootBundle.load(widget.iPath.path);
    setImageBytes(imageBytes);
  }

  Future<void> loadSnapshotBytes() async {
    RenderRepaintBoundary boxPaint = paintKey.currentContext.findRenderObject();
    ui.Image capture = await boxPaint.toImage();
    ByteData imageBytes =
    await capture.toByteData(format: ui.ImageByteFormat.png);
    setImageBytes(imageBytes);
    capture.dispose();
  }

  void setImageBytes(ByteData imageBytes) {
    List<int> values = imageBytes.buffer.asUint8List();
    photo = null;
    photo = img.decodeImage(values);
  }
}

// image lib uses uses KML color format, convert #AABBGGRR to regular #AARRGGBB
int abgrToArgb(int argbColor) {
  int r = (argbColor >> 16) & 0xFF;
  int b = argbColor & 0xFF;
  return (argbColor & 0xFF00FF00) | (b << 16) | r;
}