import 'package:fitton/component/clipper.dart';
import 'package:fitton/screens/home/home.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'dart:io';
import 'dart:ui' as ui;

import 'package:image_picker/image_picker.dart';
import 'package:firebase_ml_vision/firebase_ml_vision.dart';
import 'package:fitton/component/default_button.dart';
import 'package:fitton/component/notfound_comp.dart';
import 'package:fitton/screens/skinColourDetection/colourdetector.dart';

class ImgPicker extends StatefulWidget {
  @override
  _ImgPickerState createState() => _ImgPickerState(iDetails: null);
}

class _ImgPickerState extends State<ImgPicker> {

  File _image;
  List<Face> _face;
  int faceCount = 0;
  bool isLoad = false;
  bool isContinue = false;
  ui.Image _img;
  final ImageDetails iDetails;

  _ImgPickerState({ @required this.iDetails});

  @override
  void initState() {
    super.initState();
    setState(() {
      _image = null;
    });
  }

  final imagePicker = ImagePicker();
  Future getImage() async {

    final imageFile = File(await ImagePicker().getImage(source: ImageSource.gallery).then((pickedFile) => pickedFile.path));
    setState(() {
      _image = File(imageFile.path);
      isLoad = true;
      isContinue = false;
    });

    final FirebaseVisionImage visionImage = FirebaseVisionImage.fromFile(imageFile);
    final FaceDetector faceDetector = FirebaseVision.instance.faceDetector();
    final List<Face> faces = await faceDetector.processImage(visionImage);
    if(mounted) {
      setState(() {
        _face = faces;
        faceCount = faces.length;
        _loadImage(imageFile);
      });
    }
  }

  _loadImage(File file) async {
    final data = await file.readAsBytes();
    await decodeImageFromList(data).then(
          (value) => setState(() {
        _img = value;
        isLoad = false;
      }),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Stack(
        children: [
          ClipPath(
              clipper: MyClipper(),
              child: Container(
                color: Colors.deepPurpleAccent,
              )
          ),
          SingleChildScrollView(
          child:  Column(
          children: <Widget>[
            SizedBox(
              height: 100.0,
            ),
            Center(
              child: Text("Capture your Selfie", style: TextStyle(color: Colors.white, fontSize: 35, fontWeight: FontWeight.bold),),
            ),
            SizedBox(
              height: 60.0,
            ),
            Container(
              width: 350,
              height: 350,
              child: _image == null ?
                     Image.asset('assets/images/tookimg.png',height: 90.0,) :
                     isLoad ? Center(
                         child: CircularProgressIndicator()) :
                     Center(
                       child: _face.length == 0 ?
                       NotFound(text: "Ooops! Can't Find a Face",):
                       FittedBox(
                         child: SizedBox(
                           width: _img.width.toDouble(),
                           height: _img.height.toDouble(),
                           child: CustomPaint(
                             painter: FacePainter(_img, _face),
                           ),
                         ),
                       ),
                     ),
              decoration: BoxDecoration(
                  color: Colors.grey[300],
                  borderRadius: BorderRadius.all(Radius.circular(50)),
                  boxShadow: [
                    BoxShadow(
                        color: Colors.grey[400],
                        offset: Offset(-4.0, -4.0,),
                        blurRadius: 15.0,
                        spreadRadius: 1.0
                    ),
                  ]),
            ),
            Padding(
              padding: EdgeInsets.all(30.0),
              child: Column(
                  children: <Widget>[
                    SizedBox(
                      height: 60.0,
                    ),
                    DefaultButton(
                      text: _image == null ? 'Take a Selfie' : 'Try Another',
                      bg: true,
                      press: getImage,
                    ),
                    SizedBox(
                      height: 30.0,
                    ),
                    DefaultButton(
                      text: faceCount == 0 ? "Back" :"Continue the Magic ✨",
                      bg: false,
                      press: faceCount == 0 ? () {
                        Navigator.push(
                          context,
                          MaterialPageRoute(builder: (context) => (Home())),
                        );
                      } :() {
                        Navigator.push(
                          context,
                          MaterialPageRoute(builder: (context) => (ColorPickerWidget(_image))),
                        );
                      },
                    ),
                  ]
              ),
            ),
          ],
          ),
        ),
      ]
      ),
    );
  }
}

// Detect face part and mark it
class FacePainter extends CustomPainter {
  final ui.Image image;
  final List<Face> faces;
  final List<Rect> rects = [];

  FacePainter(this.image, this.faces) {
    for (var i = 0; i < faces.length; i++) {
      rects.add(faces[i].boundingBox);
    }
  }

  @override
  void paint(ui.Canvas canvas, ui.Size size) {
    final Paint paint = Paint()
      ..style = PaintingStyle.stroke
      ..strokeWidth = 15.0
      ..color = Colors.yellow;

    canvas.drawImage(image, Offset.zero, Paint());
    for (var i = 0; i < faces.length; i++) {
      canvas.drawRect(rects[i], paint);
    }
  }

  @override
  bool shouldRepaint(FacePainter oldDelegate) {
    return image != oldDelegate.image || faces != oldDelegate.faces;
  }
}

class ImageDetails{
  final String iPath;
  ImageDetails(this.iPath);
}